@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.h,v 1.1.1.9 2006-08-11 18:38:22 rousseau Exp $
+ * $Id: ccid.h,v 1.8 2007-01-27 17:06:49 rousseau Exp $
  */
 
 typedef struct
@@ -78,7 +78,7 @@ typedef struct
 	 * The array of data rates supported by the reader
 	 */
 	unsigned int *arrayOfSupportedDataRates;
-	
+
 	/*
 	 * Read communication port timeout
 	 * value is seconds
@@ -138,6 +138,13 @@ typedef struct
 #define CHERRYXX44	0x046a0010
 #define CL1356D		0x0B810200
 
+/*
+ * The O2Micro OZ776S reader has a wrong USB descriptor
+ * The extra[] field is associated with the last endpoint instead of the
+ * main USB descriptor
+ */
+#define O2MICRO_OZ776_PATCH
+
 /* Escape sequence codes */
 #define ESC_GEMPC_SET_ISO_MODE		1
 #define ESC_GEMPC_SET_APDU_MODE		2
@@ -160,7 +167,7 @@ typedef struct
 #define VOLTAGE_1_8V 3
 
 int ccid_open_hack(unsigned int reader_index);
-void ccid_error(int error, char *file, int line, const char *function);
+void ccid_error(int error, const char *file, int line, const char *function);
 _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index);
 
 /* convert a 4 byte integer in USB format into an int */
