@@ -18,13 +18,13 @@
 */
 
 /*
- * $Id: ccid_usb.c 2119 2006-08-01 20:05:22Z rousseau $
+ * $Id: ccid_usb.c 2359 2007-01-27 13:31:33Z rousseau $
  */
 
 #define __CCID_USB__
 
 #include <stdio.h>
-#include <string.h> 
+#include <string.h>
 #include <errno.h>
 # ifdef S_SPLINT_S
 # include <sys/types.h>
@@ -53,19 +53,13 @@
  * Manufacturer and Product ID are also used to identify the device */
 #define ALLOW_PROPRIETARY_CLASS
 
-/*
- * The O2Micro OZ776S reader has a wrong USB descriptor
- * The extra[] field is associated with the last endpoint instead of the
- * main USB descriptor
- */
-#define O2MICRO_OZ776_PATCH
-
 #define BUS_DEVICE_STRSIZE 32
 
 typedef struct
 {
 	usb_dev_handle *handle;
-	struct usb_device *dev;
+	char *dirname;
+	char *filename;
 	int interface;
 
 	/*
@@ -90,7 +84,8 @@ typedef struct
 
 static int get_end_points(struct usb_device *dev, _usbDevice *usb_device);
 int ccid_check_firmware(struct usb_device *dev);
-static unsigned int *get_data_rates(unsigned int reader_index);
+static unsigned int *get_data_rates(unsigned int reader_index,
+	struct usb_device *dev);
 
 /* ne need to initialize to 0 since it is static */
 static _usbDevice usbDevice[CCID_DRIVER_MAX_READERS];
@@ -107,7 +102,9 @@ struct _bogus_firmware
 };
 
 static struct _bogus_firmware Bogus_firmwares[] = {
+#ifndef O2MICRO_OZ776_PATCH
 	{ 0x0b97, 0x7762, 0x0111 },	/* Oz776S */ /* the firmware version is not correct since I do not have received a working reader yet */
+#endif
 	{ 0x04e6, 0xe001, 0x0516 },	/* SCR 331 */
 	{ 0x04e6, 0x5111, 0x0620 },	/* SCR 331-DI */
 	{ 0x04e6, 0x5115, 0x0514 },	/* SCR 335 */
@@ -117,6 +114,7 @@ static struct _bogus_firmware Bogus_firmwares[] = {
 	{ 0x09C3, 0x0008, 0x0203 },	/* ActivCard V2 */
 	{ 0x0DC3, 0x1004, 0x0502 },	/* ASE IIIe USBv2 */
 	{ 0x0DC3, 0x1102, 0x0607 },	/* ASE IIIe KB USB */
+	{ 0x058F, 0x9520, 0x0102 },	/* Alcor AU9520-G */
 };
 
 /* data rates supported by the secondary slots on the GemCore Pos Pro & SIM Pro */
@@ -296,14 +294,14 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					/* is it already opened? */
 					already_used = FALSE;
 
+					DEBUG_COMM3("Checking device: %s/%s",
+						bus->dirname, dev->filename);
 					for (r=0; r<CCID_DRIVER_MAX_READERS; r++)
 					{
-						if (usbDevice[r].dev)
+						if (usbDevice[r].handle)
 						{
-							DEBUG_COMM3("Checking device: %s/%s",
-								bus->dirname, dev->filename);
 							/* same busname, same filename */
-							if (strcmp(usbDevice[r].dev->bus->dirname, bus->dirname) == 0 && strcmp(usbDevice[r].dev->filename, dev->filename) == 0)
+							if (strcmp(usbDevice[r].dirname, bus->dirname) == 0 && strcmp(usbDevice[r].filename, dev->filename) == 0)
 								already_used = TRUE;
 						}
 					}
@@ -312,9 +310,9 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					if (already_used)
 					{
 						if ((previous_reader_index != -1)
-							&& usbDevice[previous_reader_index].dev
-							&& (strcmp(usbDevice[previous_reader_index].dev->bus->dirname, bus->dirname)  == 0)
-							&& (strcmp(usbDevice[previous_reader_index].dev->filename, dev->filename) == 0)
+							&& usbDevice[previous_reader_index].handle
+							&& (strcmp(usbDevice[previous_reader_index].dirname, bus->dirname)  == 0)
+							&& (strcmp(usbDevice[previous_reader_index].filename, dev->filename) == 0)
 							&& usbDevice[previous_reader_index].ccid.bCurrentSlotIndex < usbDevice[previous_reader_index].ccid.bMaxSlotIndex)
 						{
 							/* we reuse the same device
@@ -372,7 +370,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 						DEBUG_CRITICAL3("Can't find a CCID interface on %s/%s",
 							bus->dirname, dev->filename);
 						return STATUS_UNSUCCESSFUL;
-					}			
+					}
 
 					if (usb_interface->altsetting->extralen != 54)
 					{
@@ -408,7 +406,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 
 					/* store device information */
 					usbDevice[reader_index].handle = dev_handle;
-					usbDevice[reader_index].dev = dev;
+					usbDevice[reader_index].dirname = strdup(bus->dirname);
+					usbDevice[reader_index].filename = strdup(dev->filename);
 					usbDevice[reader_index].interface = interface;
 					usbDevice[reader_index].real_nb_opened_slots = 1;
 					usbDevice[reader_index].nb_opened_slots = &usbDevice[reader_index].real_nb_opened_slots;
@@ -428,7 +427,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					usbDevice[reader_index].ccid.bMaxSlotIndex = usb_interface->altsetting->extra[4];
 					usbDevice[reader_index].ccid.bCurrentSlotIndex = 0;
 					usbDevice[reader_index].ccid.readTimeout = DEFAULT_COM_READ_TIMEOUT;
-					usbDevice[reader_index].ccid.arrayOfSupportedDataRates = get_data_rates(reader_index);
+					usbDevice[reader_index].ccid.arrayOfSupportedDataRates = get_data_rates(reader_index, dev);
 					goto end;
 				}
 			}
@@ -467,14 +466,12 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 
 	if (rv < 0)
 	{
-		if (usbDevice[reader_index].dev->bus)
-		{
-			DEBUG_CRITICAL4("usb_bulk_write(%s/%s): %s",
-				usbDevice[reader_index].dev->bus->dirname,
-				usbDevice[reader_index].dev->filename, strerror(errno));
-		}
-		else
-			DEBUG_CRITICAL2("usb_bulk_write(no device): %s", strerror(errno));
+		DEBUG_CRITICAL4("usb_bulk_write(%s/%s): %s",
+			usbDevice[reader_index].dirname, usbDevice[reader_index].filename,
+			strerror(errno));
+
+		if (ENODEV == errno)
+			return STATUS_NO_SUCH_DEVICE;
 
 		return STATUS_UNSUCCESSFUL;
 	}
@@ -505,14 +502,12 @@ read_again:
 	if (rv < 0)
 	{
 		*length = 0;
-		if (usbDevice[reader_index].dev->bus)
-		{
-			DEBUG_CRITICAL4("usb_bulk_read(%s/%s): %s",
-					usbDevice[reader_index].dev->bus->dirname,
-					usbDevice[reader_index].dev->filename, strerror(errno));
-		}
-		else
-			DEBUG_CRITICAL2("usb_bulk_read(no device): %s", strerror(errno));
+		DEBUG_CRITICAL4("usb_bulk_read(%s/%s): %s",
+			usbDevice[reader_index].dirname, usbDevice[reader_index].filename,
+			strerror(errno));
+
+		if (ENODEV == errno)
+			return STATUS_NO_SUCH_DEVICE;
 
 		return STATUS_UNSUCCESSFUL;
 	}
@@ -541,12 +536,12 @@ read_again:
 status_t CloseUSB(unsigned int reader_index)
 {
 	/* device not opened */
-	if (usbDevice[reader_index].dev == NULL)
+	if (usbDevice[reader_index].handle == NULL)
 		return STATUS_UNSUCCESSFUL;
 
 	DEBUG_COMM3("Closing USB device: %s/%s",
-		usbDevice[reader_index].dev->bus->dirname,
-		usbDevice[reader_index].dev->filename);
+		usbDevice[reader_index].dirname,
+		usbDevice[reader_index].filename);
 
 	if (usbDevice[reader_index].ccid.arrayOfSupportedDataRates
 		&& (usbDevice[reader_index].ccid.bCurrentSlotIndex == 0))
@@ -570,11 +565,15 @@ status_t CloseUSB(unsigned int reader_index)
 		usb_release_interface(usbDevice[reader_index].handle,
 			usbDevice[reader_index].interface);
 		usb_close(usbDevice[reader_index].handle);
+
+		free(usbDevice[reader_index].dirname);
+		free(usbDevice[reader_index].filename);
 	}
 
 	/* mark the resource unused */
 	usbDevice[reader_index].handle = NULL;
-	usbDevice[reader_index].dev = NULL;
+	usbDevice[reader_index].dirname = NULL;
+	usbDevice[reader_index].filename = NULL;
 	usbDevice[reader_index].interface = 0;
 
 	return STATUS_SUCCESS;
@@ -592,26 +591,6 @@ _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index)
 } /* get_ccid_descriptor */
 
 
-/*****************************************************************************
- *
- *					get_desc
- *
- ****************************************************************************/
-int get_desc(int lun, usb_dev_handle **handle, struct
-	usb_device **dev)
-{
-	int reader_index;
-
-	if (-1 == (reader_index = LunToReaderIndex(lun)))
-		return FALSE;
-
-	*handle = usbDevice[reader_index].handle;
-	*dev = usbDevice[reader_index].dev;
-
-	return TRUE;
-} /* get_desc */
-
-
 /*****************************************************************************
  *
  *					get_end_points
@@ -622,7 +601,7 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usb_device)
 	int i;
 	int bEndpointAddress;
 	struct usb_interface *usb_interface = get_ccid_usb_interface(dev);
-	
+
 	/*
 	 * 3 Endpoints maximum: Interrupt In, Bulk In, Bulk Out
 	 */
@@ -651,7 +630,7 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usb_device)
  ****************************************************************************/
 /*@null@*/ EXTERNAL struct usb_interface * get_ccid_usb_interface(struct usb_device *dev)
 {
-	struct usb_interface *usb_interface = NULL; 
+	struct usb_interface *usb_interface = NULL;
 	int i;
 
 	/* if multiple interfaces use the first one with CCID class type */
@@ -740,7 +719,8 @@ int ccid_check_firmware(struct usb_device *dev)
  *                                      get_data_rates
  *
  ****************************************************************************/
-static unsigned int *get_data_rates(unsigned int reader_index)
+static unsigned int *get_data_rates(unsigned int reader_index,
+	struct usb_device *dev)
 {
 	int n, i, len;
 	unsigned char buffer[256*sizeof(int)];	/* maximum is 256 records */
@@ -775,8 +755,7 @@ static unsigned int *get_data_rates(unsigned int reader_index)
 	n /= sizeof(int);
 
 	/* we do not get the expected number of data rates */
-	len = get_ccid_usb_interface(usbDevice[reader_index].dev)
-		->altsetting->extra[27]; /* bNumDataRatesSupported */
+	len = get_ccid_usb_interface(dev)->altsetting->extra[27]; /* bNumDataRatesSupported */
 	if ((n != len) && len)
 	{
 		DEBUG_INFO3("Got %d data rates but was expecting %d", n, len);
