@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: commands.c 2120 2006-08-03 10:04:01Z rousseau $
+ * $Id: commands.c 2358 2007-01-27 13:28:26Z rousseau $
  */
 
 #include <string.h>
@@ -104,6 +104,12 @@ again:
 	if (res != STATUS_SUCCESS)
 		return IFD_COMMUNICATION_ERROR;
 
+	if (*nlength < STATUS_OFFSET+1)
+	{
+		DEBUG_CRITICAL2("Not enough data received: %d bytes", *nlength);
+		return IFD_COMMUNICATION_ERROR;
+	}
+
 	if (buffer[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
@@ -130,7 +136,7 @@ again:
 		/* continue with 3 volts and 5 volts */
 		if (voltage > 1)
 		{
-			char *voltage_code[] = { "auto", "5V", "3V", "1.8V" };
+			const char *voltage_code[] = { "auto", "5V", "3V", "1.8V" };
 
 			DEBUG_INFO3("Power up with %s failed. Try with %s.",
 				voltage_code[voltage], voltage_code[voltage-1]);
@@ -221,6 +227,16 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 				TxBuffer[8]);
 			TxBuffer[8] = 0x01;
 		}
+
+		/* The reader does not support, and actively reject, "max size reached"
+		 * and "timeout occured" validation conditions */
+		if (0x02 != TxBuffer[7])
+		{
+			DEBUG_INFO2("Correct bEntryValidationCondition for GemPC Pinpad (was %d)",
+				TxBuffer[7]);
+			TxBuffer[7] = 0x02;	/* validation key pressed */
+		}
+
 	}
 #endif
 
@@ -236,9 +252,9 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 			(void *)(TxBuffer + offsetof(PIN_VERIFY_STRUCTURE, abData)),
 			TxLength - offsetof(PIN_VERIFY_STRUCTURE, abData));
 
-		/* Create T=1 block */  
+		/* Create T=1 block */
 		ret = t1_build(&((get_ccid_slot(reader_index))->t1),
-			sdata, 0, T1_I_BLOCK, &sbuf, NULL); 
+			sdata, 0, T1_I_BLOCK, &sbuf, NULL);
 
 		/* Increment the sequence numbers  */
 		get_ccid_slot(reader_index)->t1.ns ^= 1;
@@ -446,9 +462,9 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 			(void *)(TxBuffer + offsetof(PIN_MODIFY_STRUCTURE, abData)),
 			TxLength - offsetof(PIN_MODIFY_STRUCTURE, abData));
 
-		/* Create T=1 block */  
+		/* Create T=1 block */
 		ret = t1_build(&((get_ccid_slot(reader_index))->t1),
-			sdata, 0, T1_I_BLOCK, &sbuf, NULL); 
+			sdata, 0, T1_I_BLOCK, &sbuf, NULL);
 
 		/* Increment the sequence numbers  */
 		get_ccid_slot(reader_index)->t1.ns ^= 1;
@@ -608,6 +624,12 @@ again:
 		return IFD_COMMUNICATION_ERROR;
 	}
 
+	if (length_out < STATUS_OFFSET+1)
+	{
+		DEBUG_CRITICAL2("Not enough data received: %d bytes", length_out);
+		return IFD_COMMUNICATION_ERROR;
+	}
+
 	if (cmd_out[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(cmd_out[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
@@ -655,6 +677,12 @@ RESPONSECODE CmdPowerOff(unsigned int reader_index)
 	if (res != STATUS_SUCCESS)
 		return IFD_COMMUNICATION_ERROR;
 
+	if (length < STATUS_OFFSET+1)
+	{
+		DEBUG_CRITICAL2("Not enough data received: %d bytes", length);
+		return IFD_COMMUNICATION_ERROR;
+	}
+
 	if (cmd[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
@@ -686,15 +714,31 @@ RESPONSECODE CmdGetSlotStatus(unsigned int reader_index, unsigned char buffer[])
 
 	res = WritePort(reader_index, sizeof(cmd), cmd);
 	if (res != STATUS_SUCCESS)
+	{
+		if (STATUS_NO_SUCH_DEVICE == res)
+			return IFD_NO_SUCH_DEVICE;
 		return IFD_COMMUNICATION_ERROR;
+	}
 
 	length = SIZE_GET_SLOT_STATUS;
 	res = ReadPort(reader_index, &length, buffer);
 	if (res != STATUS_SUCCESS)
 		return IFD_COMMUNICATION_ERROR;
 
+	if (length < STATUS_OFFSET+1)
+	{
+		DEBUG_CRITICAL2("Not enough data received: %d bytes", length);
+		return IFD_COMMUNICATION_ERROR;
+	}
+
 	if (buffer[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
+#ifdef O2MICRO_OZ776_PATCH
+		/* the O2MICRO OZ 776 reader sends card absent or mute errors
+		 * when no card is inserted */
+		if (! ((OZ776 == ccid_descriptor->readerID)
+			&& (buffer[ERROR_OFFSET] == 0xFE)))
+#endif
 		ccid_error(buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
 
 		/* card absent or mute is not an communication error */
@@ -776,6 +820,7 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 {
 	unsigned char cmd[10+CMD_BUF_SIZE];	/* CCID + APDU buffer */
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	status_t ret;
 
 	cmd[0] = 0x6F; /* XfrBlock */
 	i2dw(tx_length, cmd+1);	/* APDU length */
@@ -794,7 +839,10 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 
 	memcpy(cmd+10, tx_buffer, tx_length);
 
-	if (WritePort(reader_index, 10+tx_length, cmd) != STATUS_SUCCESS)
+	ret = WritePort(reader_index, 10+tx_length, cmd);
+	if (STATUS_NO_SUCH_DEVICE == ret)
+		return IFD_NO_SUCH_DEVICE;
+	if (ret != STATUS_SUCCESS)
 		return IFD_COMMUNICATION_ERROR;
 
 	return IFD_SUCCESS;
@@ -812,16 +860,20 @@ RESPONSECODE CCID_Receive(unsigned int reader_index, unsigned int *rx_length,
 	unsigned char cmd[10+CMD_BUF_SIZE];	/* CCID + APDU buffer */
 	unsigned int length;
 	RESPONSECODE return_value = IFD_SUCCESS;
+	status_t ret;
 
 time_request:
 	length = sizeof(cmd);
-	if (ReadPort(reader_index, &length, cmd) != STATUS_SUCCESS)
+	ret = ReadPort(reader_index, &length, cmd);
+	if (ret != STATUS_SUCCESS)
 	{
 		*rx_length = 0;
+		if (STATUS_NO_SUCH_DEVICE == ret)
+			return IFD_NO_SUCH_DEVICE;
 		return IFD_COMMUNICATION_ERROR;
 	}
 
-	if (length < STATUS_OFFSET)
+	if (length < STATUS_OFFSET+1)
 	{
 		DEBUG_CRITICAL2("Not enough data received: %d bytes", length);
 		*rx_length = 0;
@@ -1150,15 +1202,42 @@ static RESPONSECODE T0ProcACK(unsigned int reader_index,
 
 		memset(tmp_buf, 0, sizeof(tmp_buf));
 
-		ret_len = remain_len;
-		return_value = CCID_Transmit(reader_index, 0, *snd_buf, ret_len, 0);
-		if (return_value != IFD_SUCCESS)
-			return return_value;
+#ifdef O2MICRO_OZ776_PATCH
+		if((0 != remain_len) && (0 == (remain_len + 10) % 64))
+        {
+			/* special hack to avoid a command of size modulo 64
+			 * we send two commands instead */
+            ret_len = 1;
+            return_value = CCID_Transmit(reader_index, 0, *snd_buf, ret_len, 0);
+            if (return_value != IFD_SUCCESS)
+                return return_value;
+            return_value = CCID_Receive(reader_index, &ret_len, tmp_buf, NULL);
+            if (return_value != IFD_SUCCESS)
+                return return_value;
 
-		return_value = CCID_Receive(reader_index, &ret_len, tmp_buf, NULL);
-		if (return_value != IFD_SUCCESS)
-			return return_value;
+            ret_len = remain_len - 1;
+            return_value = CCID_Transmit(reader_index, 0, *snd_buf, ret_len, 0);
+            if (return_value != IFD_SUCCESS)
+                return return_value;
+            return_value = CCID_Receive(reader_index, &ret_len, &tmp_buf[1],
+				NULL);
+            if (return_value != IFD_SUCCESS)
+                return return_value;
 
+            ret_len += 1;
+        }
+        else
+#endif
+		{
+			ret_len = remain_len;
+			return_value = CCID_Transmit(reader_index, 0, *snd_buf, ret_len, 0);
+			if (return_value != IFD_SUCCESS)
+				return return_value;
+
+			return_value = CCID_Receive(reader_index, &ret_len, tmp_buf, NULL);
+			if (return_value != IFD_SUCCESS)
+				return return_value;
+		}
 		memcpy(*rcv_buf, tmp_buf, remain_len);
 		*rcv_buf += remain_len, *rcv_len += remain_len;
 
@@ -1438,6 +1517,12 @@ RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 	if (ReadPort(reader_index, &length, cmd) != STATUS_SUCCESS)
 		return IFD_COMMUNICATION_ERROR;
 
+	if (length < STATUS_OFFSET+1)
+	{
+		DEBUG_CRITICAL2("Not enough data received: %d bytes", length);
+		return IFD_COMMUNICATION_ERROR;
+	}
+
 	if (cmd[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
