@@ -17,7 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
-/* $Id: ifdhandler.c 2130 2006-08-09 07:17:31Z rousseau $ */
+/* $Id: ifdhandler.c 2258 2006-12-02 17:13:06Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -60,7 +60,7 @@ static int DebugInitialized = FALSE;
 static void init_driver(void);
 static void extra_egt(ATR_t *atr, _ccid_descriptor *ccid_desc, DWORD Protocol);
 static char find_baud_rate(unsigned int baudrate, unsigned int *list);
-static unsigned int T0_card_timeout(double f, double d, int TC1, int TC2, 
+static unsigned int T0_card_timeout(double f, double d, int TC1, int TC2,
 	int clock_frequency);
 static unsigned int T1_card_timeout(double f, double d, int TC1, int BWI,
 	int CWI, int clock_frequency);
@@ -322,6 +322,28 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 			}
 			break;
 
+		case SCARD_ATTR_VENDOR_IFD_VERSION:
+			/* Vendor-supplied interface device version (DWORD in the form
+			 * 0xMMmmbbbb where MM = major version, mm = minor version, and
+			 * bbbb = build number). */
+			*Length = sizeof(DWORD);
+			if (Value)
+				*(DWORD *)Value = CCID_VERSION;
+			break;
+
+		case SCARD_ATTR_VENDOR_NAME:
+#define VENDOR_NAME "Ludovic Rousseau"
+			*Length = sizeof(VENDOR_NAME);
+			if (Value)
+				memcpy(Value, VENDOR_NAME, sizeof(VENDOR_NAME));
+			break;
+
+		case SCARD_ATTR_MAXINPUT:
+			*Length = sizeof(uint32_t);
+			if (Value)
+				*(uint32_t *)Value = get_ccid_descriptor(reader_index) -> dwMaxCCIDMessageLength -10;
+			break;
+
 		default:
 			return IFD_ERROR_TAG;
 	}
@@ -672,6 +694,18 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			(param[3] & 0xF0) >> 4 /* BWI */, param[3] & 0x0F /* CWI */,
 			ccid_desc->dwDefaultClock);
 
+		/* TAi (i>2) present? IFSC */
+		for (i=2; i<ATR_MAX_PROTOCOLS; i++)
+			if (atr.ib[i][ATR_INTERFACE_BYTE_TA].present)
+			{
+				DEBUG_COMM3("IFSC (TA%d) present: %d", i+1,
+					atr.ib[i][ATR_INTERFACE_BYTE_TA].value);
+				 param[5] = atr.ib[i][ATR_INTERFACE_BYTE_TA].value;
+
+				/* only the first TAi (i>2) must be used */
+				break;
+			}
+
 		DEBUG_COMM2("Timeout: %d seconds", ccid_desc->readTimeout);
 
 		ret = SetParameters(reader_index, 1, sizeof(param), param);
@@ -802,8 +836,9 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 	RESPONSECODE return_value = IFD_SUCCESS;
 	unsigned char pcbuffer[RESP_BUF_SIZE];
 	int reader_index;
+	const char *actions[] = { "PowerUp", "PowerDown", "Reset" };
 
-	DEBUG_INFO2("lun: %X", Lun);
+	DEBUG_INFO3("lun: %X, action: %s", Lun, actions[Action-IFD_POWER_UP]);
 
 	/* By default, assume it won't work :) */
 	*AtrLength = 0;
@@ -926,7 +961,7 @@ EXTERNAL RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 
 EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 	PUCHAR TxBuffer, DWORD TxLength, PUCHAR RxBuffer, DWORD RxLength,
-	PDWORD pdwBytesReturned) 
+	PDWORD pdwBytesReturned)
 {
 	/*
 	 * This function performs a data exchange with the reader (not the
@@ -973,7 +1008,7 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 	}
 
 	/* Implement the PC/SC v2.1.2 Part 10 IOCTL mechanism */
-	
+
 	/* Query for features */
 	if (CM_IOCTL_GET_FEATURE_REQUEST == dwControlCode)
 	{
@@ -995,7 +1030,7 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 			pcsc_tlv++;
 			iBytesReturned += sizeof(PCSC_TLV_STRUCTURE);
 		}
-		
+
 		if (get_ccid_descriptor(reader_index) -> bPINSupport
 			& CCID_CLASS_PIN_MODIFY)
 		{
@@ -1080,17 +1115,31 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 	LogLevel = oldLogLevel;
 
 	if (return_value != IFD_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+		return return_value;
 
 	return_value = IFD_COMMUNICATION_ERROR;
 	switch (pcbuffer[7] & CCID_ICC_STATUS_MASK)	/* bStatus */
 	{
 		case CCID_ICC_PRESENT_ACTIVE:
-		case CCID_ICC_PRESENT_INACTIVE:
 			return_value = IFD_ICC_PRESENT;
 			/* use default slot */
 			break;
 
+		case CCID_ICC_PRESENT_INACTIVE:
+			if ((CcidSlots[reader_index].bPowerFlags == POWERFLAGS_RAZ)
+				|| (CcidSlots[reader_index].bPowerFlags & MASK_POWERFLAGS_PDWN))
+				/* the card was previously absent */
+				return_value = IFD_ICC_PRESENT;
+			else
+			{
+				/* the card was previously present but has been
+				 * removed and inserted between two consecutive
+				 * IFDHICCPresence() calls */
+				CcidSlots[reader_index].bPowerFlags = POWERFLAGS_RAZ;
+				return_value = IFD_ICC_NOT_PRESENT;
+			}
+			break;
+
 		case CCID_ICC_ABSENT:
 			/* Reset ATR buffer */
 			CcidSlots[reader_index].nATRLength = 0;
@@ -1134,7 +1183,7 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 			/* simulate a card absent */
 			res[0] = 0;
 		}
-		
+
 		if (0x01 == res[0])
 			return_value = IFD_ICC_PRESENT;
 		else
@@ -1278,7 +1327,7 @@ void extra_egt(ATR_t *atr, _ccid_descriptor *ccid_desc, DWORD Protocol)
 			for (i=2; i<ATR_MAX_PROTOCOLS; i++)
 			{
 				/* CWI >= 2 ? */
-				if (atr->ib[i][ATR_INTERFACE_BYTE_TB].present && 
+				if (atr->ib[i][ATR_INTERFACE_BYTE_TB].present &&
 					((atr->ib[i][ATR_INTERFACE_BYTE_TB].value & 0x0F) >= 2))
 				{
 					/* Init TC1 */
@@ -1343,7 +1392,7 @@ static unsigned int T0_card_timeout(double f, double d, int TC1, int TC2,
 	 * Terminal:                    Smart card:
 	 * 5 bytes header cmd  ->
 	 * 					  <-        Procedure byte + 256 data bytes + SW1-SW2
-	 * = 5 EGT          + 1 WWT     + 259 WWT			
+	 * = 5 EGT          + 1 WWT     + 259 WWT
 	 */
 
 	/* clock_frequency is in kHz so the times are in milliseconds and not
@@ -1389,11 +1438,11 @@ static unsigned int T1_card_timeout(double f, double d, int TC1,
 	/* Timeout applied on ISO in + ISO out card exchange
 	 *
      * Timeout is the sum of:
-	 * - ISO in delay between leading edge of the first character sent by the 
+	 * - ISO in delay between leading edge of the first character sent by the
 	 *   interface device and the last one (NAD PCB LN APDU CKS) = 260 EGT,
 	 * - delay between ISO in and ISO out = BWT,
-	 * - ISO out delay between leading edge of the first character sent by the 
-	 *   card and the last one (NAD PCB LN DATAS CKS) = 260 CWT.   
+	 * - ISO out delay between leading edge of the first character sent by the
+	 *   card and the last one (NAD PCB LN DATAS CKS) = 260 CWT.
 	 */
 
 	/* clock_frequency is in kHz so the times are in milliseconds and not
