@@ -18,11 +18,11 @@
 */
 
 /*
- * $Id: parse.c 1819 2006-01-20 13:27:21Z rousseau $
+ * $Id: parse.c 2328 2007-01-10 20:53:44Z rousseau $
  */
 
 #include <stdio.h>
-#include <string.h> 
+#include <string.h>
 # ifdef S_SPLINT_S
 # include <sys/types.h>
 # endif
@@ -46,7 +46,7 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
  *					main
  *
  ****************************************************************************/
-int main(/*@unused@*/ int argc, /*@unused@*/ char *argv[])
+int main(void)
 {
 	static struct usb_bus *busses = NULL;
 	struct usb_bus *bus;
@@ -59,7 +59,7 @@ int main(/*@unused@*/ int argc, /*@unused@*/ char *argv[])
 	busses = usb_get_busses();
 	if (busses == NULL)
 	{
-		printf("No USB busses found");
+		printf("No USB buses found\n");
 		return -1;
 	}
 
@@ -105,7 +105,7 @@ int main(/*@unused@*/ int argc, /*@unused@*/ char *argv[])
 				fprintf(stderr, "Can't find a CCID interface on %s/%s\n",
 					bus->dirname, dev->filename);
 				continue;
-			}			
+			}
 
 			ccid_parse_interface_descriptor(dev_handle, dev);
 			usb_close(dev_handle);
@@ -134,7 +134,14 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 	printf(" idVendor: 0x%04X\n", dev->descriptor.idVendor);
 	if (usb_get_string_simple(handle, dev->descriptor.iManufacturer,
 		buffer, sizeof(buffer)) < 0)
+	{
 		printf("  Can't get iManufacturer string\n");
+		if (getuid())
+		{
+			printf("\33[01;31mPlease, restart the command as root\33[0m\n\n");
+			return TRUE;
+		}
+	}
 	else
 		printf("  iManufacturer: %s\n", buffer);
 
@@ -149,15 +156,15 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 		dev->descriptor.bcdDevice >> 8, dev->descriptor.bcdDevice & 0xFF);
 
 	usb_interface = get_ccid_usb_interface(dev)->altsetting;
-	
+
 	printf(" bLength: %d\n", usb_interface->bLength);
-	
+
 	printf(" bDescriptorType: %d\n", usb_interface->bDescriptorType);
-	
+
 	printf(" bInterfaceNumber: %d\n", usb_interface->bInterfaceNumber);
-	
+
 	printf(" bAlternateSetting: %d\n", usb_interface->bAlternateSetting);
-	
+
 	printf(" bNumEndpoints: %d\n", usb_interface->bNumEndpoints);
 
 	printf(" bInterfaceClass: 0x%02X", usb_interface->bInterfaceClass);
@@ -171,7 +178,7 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 		else
 			printf("  Class is 0xFF (proprietary)\n");
 	}
-	
+
 	printf(" bInterfaceSubClass: %d\n", usb_interface->bInterfaceSubClass);
 	if (usb_interface->bInterfaceSubClass)
 		printf("  UNSUPPORTED SubClass\n");
@@ -319,11 +326,11 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 	}
 	printf("  dwMaxIFSD: %d\n", dw2i(extra, 28));
 	printf("  dwSynchProtocols: 0x%08X\n", dw2i(extra, 32));
-	if (extra[35] & 0x01)
+	if (extra[32] & 0x01)
 			printf("   2-wire protocol\n");
-	if (extra[35] & 0x02)
+	if (extra[32] & 0x02)
 			printf("   3-wire protocol\n");
-	if (extra[35] & 0x04)
+	if (extra[32] & 0x04)
 			printf("   I2C protocol\n");
 
 	printf("  dwMechanical: 0x%08X\n", dw2i(extra, 36));
